import threading
from subprocess import call

import codecs
import os
from flask import Flask, request, redirect, url_for, render_template, flash

# create our little application :)
from issues_search import Search


"""
app.py Flow:

routes:
    - main route redirects user to search route
    - search route uses search.html template
        - it creates new Search object and calls search()
        - see search.py
    - update_index route creates/updates search index
        - it creates new Search object and calls update_index_incremental
        - see search.py
"""

def get_items():
    repo_list = app.config["REPOS"]
    repo_org =app.config["REPO_ORG"]
    
    gh_access_token = os.environ['GITHUB_TOKEN']

    return repo_list, repo_org, gh_access_token


class UpdateIndexTask(object):
    def __init__(self, rebuild_index=False):
        self.rebuild_index = rebuild_index
        thread = threading.Thread(target=self.run, args=())
        thread.daemon = True
        thread.start()

    def run(self, rebuild_index=False):
        search = Search(app.config["INDEX_DIR"])
        repo_list, repo_org, gh_access_token = get_items()
        search.update_index_incremental(gh_access_token, repo_list, repo_org,
                                        app.config, create_new_index=self.rebuild_index)

app = Flask(__name__)

# Load default config and override config from an environment variable
app.config.from_pyfile("config.py")

last_searches_file = app.config["INDEX_DIR"] + "/last_searches.txt"
directories_file = app.config["INDEX_DIR"] + "/directories.txt"

@app.route('/')
def index():
    return redirect(url_for("search", query="", fields=""))

@app.route('/search')
def search():
    query = request.args['query']
    fields = request.args.get('fields')
    if fields == 'None':
        fields = None

    search = Search(app.config["INDEX_DIR"])
    if not query:
        parsed_query = ""
        result = []

    else:
        parsed_query, result = search.search(query.split(), fields=[fields])
        store_search(query, fields)

    total = search.get_document_total_count()

    return render_template('search.html', entries=result, query=query, parsed_query=parsed_query, fields=fields, last_searches=get_last_searches(), total=total)

@app.route('/open')
def open_file():
    path = request.args['path']
    fields = request.args.get('fields')
    query = request.args['query']
    call([app.config["EDIT_COMMAND"], path])

    return redirect(url_for("search", query=query, fields=fields))

@app.route('/update_index')
def update_index():
    rebuild = request.args.get('rebuild')
    if rebuild:
        UpdateIndexTask(rebuild_index=True)
        flash("Rebuilding index, check console output")
    else:
        UpdateIndexTask()
        flash("Updating index, check console output")
    return render_template("search.html", query="", fields="", last_searches=get_last_searches())


def get_last_searches():
    if os.path.exists(last_searches_file):
        with codecs.open(last_searches_file, 'r', encoding='utf-8') as f:
            contents = f.readlines()
    else:
        contents = []
    return contents


def store_search(query, fields):
    if os.path.exists(last_searches_file):
        with codecs.open(last_searches_file, 'r', encoding='utf-8') as f:
            contents = f.readlines()
    else:
        contents = []

    search = "query=%s&fields=%s\n" % (query, fields)
    if not search in contents:
        contents.insert(0, search)

    with codecs.open(last_searches_file, 'w', encoding='utf-8') as f:
        f.writelines(contents[:30])


if __name__ == '__main__':
    app.run()
