#!/usr/bin/env python
from github import Github
import os
from pprint import pprint


"""
Make Markdown

This script makes a Markdown file from each Github issue
in a list of user-provided DCPPC repositories.

This uses the PyGithub library to extract a repository's
issues and comments, put the text in a markdown file, and 
add it to the repository.

TODO:
    - folder with repo name
    - markdown file named by issue/comment id
"""


list_of_repos = ['internal',
                 'organize',
                 '2018-may-workshop',
                 '2018-june-workshop',
                 '2018-july-workshop']
which_org = "dcppc"


def main():
    """
    main() is da main mane, mane
    """
    make_markdown()


def make_markdown():
    """
    Make markdown files from issues in all repos
    """

    access_token = os.environ['GITHUB_TOKEN']

    g = Github(access_token)
    org = g.get_organization(which_org)

    for this_repo in list_of_repos:
        repo = org.get_repo(this_repo)

        print("Making markdown for repo %s in %s"%(repo.name, repo2filename(repo)))
        make_markdown_from_issues(repo)
        print("Done.")


def repo2filename(repo):
    return repo.name + '.markdown'

def make_markdown_from_issues(repo):
    """
    Given a repo name,
    iterate through every issue
    and extract the issue contents
    to a markdown file.
    """
    with open(repo2filename(repo), 'w') as f:

        reponame = repo.name
        issues = repo.get_issues()

        for issue in issues:

            print("\n",         file=f)
            print("-"*10,       file=f)
            print("\n",         file=f)
            print("Issue Title: %s\n"%(issue.title),   file=f)
            print("Issue Number: %s\n"%(issue.number), file=f)
            print("Issue URL: https://github.com/%s/%s/issues/%s\n"%(
                        which_org, reponame, issue.number), file=f)
            print("Issue Labels: %s\n"%(", ".join([j.name for j in
                        issue.get_labels()])),               file=f)
            print("Issue Date: %s\n"%(issue.created_at),       file=f)
            print("Issue User: %s\n"%(issue.user.login),       file=f)
            print("Issue Body: \n%s\n"%(issue.body.rstrip()), file=f)

            if(issue.assignee is not None):
                print("Issue Assignee: %s\n"%(", ".join([j.login for j in
                    issue.assignees])),     file=f)

            if(issue.comments>0):
                comments = issue.get_comments()
                for comment in comments:
                    print("\n",     file=f)
                    print("-"*40,   file=f)
                    print("\n",     file=f)
                    print("Comment ID: %s\n"%(     comment.id),           file=f)
                    print("Comment user: %s\n"%(   comment.user.login),   file=f)
                    print("Comment URL: %s\n"%(    comment.html_url),     file=f)
                    print("Comment date: %s\n"%(   comment.created_at),   file=f)
                    print("Comment body:\n%s\n"%(comment.body.strip()),  file=f)


if __name__=="__main__":
    main()
